@@ -9,8 +9,8 @@ use crate::currency::converter::{CurrencyConverter, CurrencyConverterRc};
 use crate::formatting::table::Cell;
 use crate::localities::Country;
 use crate::quotes::Quotes;
-use crate::taxes::IncomeType;
-use crate::trades::convert_price;
+use crate::taxes::{IncomeType, long_term_ownership::LtoDeductionCalculator};
+use crate::trades;
 use crate::time;
 use crate::types::{Date, Decimal};
 use crate::util;
@@ -46,7 +46,7 @@ pub fn simulate_sell(
 
         let mut price = quotes.get(&symbol)?;
         if let Some(base_currency) = base_currency {
-            price = convert_price(price, quantity, base_currency, &converter)?;
+            price = trades::convert_price(price, quantity, base_currency, &converter)?;
         }
 
         statement.emulate_sell(&symbol, quantity, price, &mut commission_calc)?;
@@ -63,6 +63,181 @@ pub fn simulate_sell(
     print_results(country, portfolio, stock_sells, additional_commissions, &converter)
 }
 
+fn print_results(
+    country: &Country, portfolio: &PortfolioConfig,
+    stock_sells: Vec<StockSell>, additional_commissions: MultiCurrencyCashAccount,
+    converter: &CurrencyConverter,
+) -> EmptyResult {
+    let mut lto_calculator = LtoDeductionCalculator::new();
+
+    let conclusion_time = time::today_trade_conclusion_time();
+    let execution_date = time::today_trade_execution_date();
+
+    let mut total_purchase_cost = MultiCurrencyCashAccount::new();
+    let mut total_purchase_local_cost = Cash::zero(country.currency);
+
+    let mut total_revenue = MultiCurrencyCashAccount::new();
+    let mut total_local_revenue = Cash::zero(country.currency);
+
+    let mut total_profit = MultiCurrencyCashAccount::new();
+    let mut total_local_profit = Cash::zero(country.currency);
+    let mut total_taxable_local_profit = Cash::zero(country.currency);
+
+    let mut total_commission = MultiCurrencyCashAccount::new();
+
+    for commission in additional_commissions.iter() {
+        total_commission.deposit(commission.round());
+
+        let local_commission = converter.convert_to_cash_rounding(
+            conclusion_time.date, commission, country.currency)?;
+
+        total_profit.withdraw(commission);
+        total_local_profit -= local_commission;
+        total_taxable_local_profit -= local_commission;
+    }
+
+    let mut trades_table = TradesTable::new();
+    let mut fifo_table = FifoTable::new();
+
+    let mut same_currency = true;
+    let mut tax_exemptions = false;
+    let mut long_term_ownership = false;
+
+    for trade in stock_sells {
+        let (sell_price, commission) = match trade.type_ {
+            StockSellType::Trade {price, commission, ..} => {
+                same_currency &=
+                    price.currency == country.currency &&
+                    commission.currency == country.currency;
+
+                (price, commission.round())
+            },
+            _ => unreachable!(),
+        };
+        total_commission.deposit(commission);
+
+        let (tax_year, _) = portfolio.tax_payment_day().get(trade.execution_date, true);
+        let details = trade.calculate(&country, tax_year, &portfolio.tax_exemptions, &converter)?;
+        let real = details.real_profit(&converter)?;
+        tax_exemptions |= details.tax_exemption_applied();
+
+        total_purchase_cost.deposit(details.purchase_cost);
+        total_purchase_local_cost += details.purchase_local_cost;
+
+        total_revenue.deposit(details.revenue);
+        total_local_revenue += details.local_revenue;
+
+        total_profit.deposit(details.profit);
+        total_local_profit += details.local_profit;
+        total_taxable_local_profit += details.taxable_local_profit;
+
+        let price_precision = std::cmp::max(2, util::decimal_precision(sell_price.amount));
+        let mut purchase_cost = Cash::zero(sell_price.currency);
+
+        for (index, buy_trade) in details.fifo.iter().enumerate() {
+            let buy_price = buy_trade.price(sell_price.currency, converter)?;
+            purchase_cost += buy_trade.cost(purchase_cost.currency, converter)?;
+
+            if let Some(ref deductible) = buy_trade.long_term_ownership_deductible {
+                lto_calculator.add(deductible.profit, deductible.years, false);
+                long_term_ownership = true;
+            }
+
+            fifo_table.add_row(FifoRow {
+                symbol: if index == 0 {
+                   Some(trade.symbol.clone())
+                } else {
+                   None
+                },
+                date: buy_trade.conclusion_time.date,
+                quantity: (buy_trade.quantity * buy_trade.multiplier).normalize(),
+                price: (buy_price / buy_trade.multiplier).normalize(),
+                long_term_ownership: buy_trade.long_term_ownership_deductible.is_some(),
+                tax_free: buy_trade.tax_exemption_applied,
+            });
+        }
+
+        trades_table.add_row(TradeRow {
+            symbol: trade.symbol,
+            quantity: trade.quantity,
+            buy_price: (purchase_cost / trade.quantity).round_to(price_precision).normalize(),
+            sell_price,
+            commission,
+
+            revenue: details.revenue,
+            local_revenue: details.local_revenue,
+
+            profit: details.profit,
+            local_profit: details.local_profit,
+            taxable_local_profit: details.taxable_local_profit,
+
+            tax_to_pay: details.tax_to_pay,
+            tax_deduction: details.tax_deduction,
+
+            real_profit: real.profit_ratio.map(Cell::new_ratio),
+            real_tax: real.tax_ratio.map(Cell::new_ratio),
+            real_local_profit: real.local_profit_ratio.map(Cell::new_ratio),
+        });
+    }
+
+    let lto = lto_calculator.calculate();
+    total_taxable_local_profit.amount -= lto.deduction;
+
+    let (tax_year, _) = portfolio.tax_payment_day().get(execution_date, true);
+    let tax_without_deduction = country.tax_to_pay(
+        IncomeType::Trading, tax_year, total_local_profit, None);
+    let tax_to_pay = country.tax_to_pay(
+        IncomeType::Trading, tax_year, total_taxable_local_profit, None);
+
+    let tax_deduction = tax_without_deduction - tax_to_pay;
+    assert!(!tax_deduction.is_negative());
+
+    let total_real = trades::calculate_real_profit(
+        converter.real_time_date(), total_purchase_cost, total_purchase_local_cost,
+        total_profit.clone(), total_local_profit, tax_to_pay, &converter)?;
+
+    let mut totals = trades_table.add_empty_row();
+    totals.set_commission(total_commission);
+    totals.set_revenue(total_revenue);
+    totals.set_local_revenue(total_local_revenue);
+    totals.set_profit(total_profit);
+    totals.set_local_profit(total_local_profit);
+    totals.set_taxable_local_profit(total_taxable_local_profit);
+    totals.set_tax_to_pay(tax_to_pay);
+    totals.set_tax_deduction(tax_deduction);
+    totals.set_real_profit(total_real.profit_ratio.map(Cell::new_ratio));
+    totals.set_real_tax(total_real.tax_ratio.map(Cell::new_ratio));
+    totals.set_real_local_profit(total_real.local_profit_ratio.map(Cell::new_ratio));
+
+    if same_currency {
+        trades_table.hide_local_revenue();
+        trades_table.hide_local_profit();
+        trades_table.hide_real_local_profit();
+    }
+    if same_currency && !long_term_ownership {
+        trades_table.hide_real_tax();
+    }
+    if !tax_exemptions && !long_term_ownership {
+        trades_table.hide_taxable_local_profit();
+        trades_table.hide_tax_deduction();
+    }
+    if !tax_exemptions {
+        fifo_table.hide_tax_free();
+    }
+    if !long_term_ownership {
+        fifo_table.hide_long_term_ownership();
+    }
+
+    trades_table.print("Sell simulation results");
+    fifo_table.print("FIFO details");
+
+    if long_term_ownership {
+        lto.print("Long term ownership deduction");
+    }
+
+    Ok(())
+}
+
 #[derive(StaticTable)]
 #[table(name="TradesTable")]
 struct TradeRow {
@@ -113,146 +288,4 @@ struct FifoRow {
     long_term_ownership: bool,
     #[column(name="Tax free", align="center")]
     tax_free: bool,
-}
-
-fn print_results(
-    country: &Country, portfolio: &PortfolioConfig,
-    stock_sells: Vec<StockSell>, additional_commissions: MultiCurrencyCashAccount,
-    converter: &CurrencyConverter,
-) -> EmptyResult {
-    let mut same_currency = true;
-
-    let conclusion_time = time::today_trade_conclusion_time();
-    let execution_date = time::today_trade_execution_date();
-
-    let mut total_revenue = MultiCurrencyCashAccount::new();
-    let mut total_local_revenue = Cash::new(country.currency, dec!(0));
-
-    let mut total_profit = MultiCurrencyCashAccount::new();
-    let mut total_local_profit = Cash::new(country.currency, dec!(0));
-    let mut total_taxable_local_profit = Cash::new(country.currency, dec!(0));
-
-    let mut total_commission = MultiCurrencyCashAccount::new();
-    let mut total_tax_deduction = Cash::new(country.currency, dec!(0));
-
-    for commission in additional_commissions.iter() {
-        total_commission.deposit(commission.round());
-
-        let local_commission = converter.convert_to_cash_rounding(
-            conclusion_time.date, commission, country.currency)?;
-
-        total_profit.withdraw(commission);
-        total_local_profit.sub_assign(local_commission).unwrap();
-        total_taxable_local_profit.sub_assign(local_commission).unwrap();
-    }
-
-    let mut trades_table = TradesTable::new();
-    let mut fifo_table = FifoTable::new();
-    let mut long_term_ownership = false;
-    let mut tax_exemptions = false;
-
-    for trade in stock_sells {
-        let (sell_price, commission) = match trade.type_ {
-            StockSellType::Trade {price, commission, ..} => {
-                same_currency &=
-                    price.currency == country.currency &&
-                    commission.currency == country.currency;
-
-                (price, commission.round())
-            },
-            _ => unreachable!(),
-        };
-        total_commission.deposit(commission);
-
-        let (tax_year, _) = portfolio.tax_payment_day().get(trade.execution_date, true);
-        let details = trade.calculate(&country, tax_year, &portfolio.tax_exemptions, &converter)?;
-        long_term_ownership |= details.long_term_ownership();
-        tax_exemptions |= details.tax_exemption_applied();
-
-        total_revenue.deposit(details.revenue);
-        total_local_revenue.add_assign(details.local_revenue).unwrap();
-
-        total_profit.deposit(details.profit);
-        total_local_profit.add_assign(details.local_profit).unwrap();
-        total_taxable_local_profit.add_assign(details.taxable_local_profit).unwrap();
-
-        total_tax_deduction.add_assign(details.tax_deduction).unwrap();
-
-        let price_precision = std::cmp::max(2, util::decimal_precision(sell_price.amount));
-        let mut purchase_cost = Cash::new(sell_price.currency, dec!(0));
-
-        for (index, buy_trade) in details.fifo.iter().enumerate() {
-            let buy_price = buy_trade.price(sell_price.currency, converter)?;
-            purchase_cost.add_assign(buy_trade.cost(purchase_cost.currency, converter)?).unwrap();
-
-            fifo_table.add_row(FifoRow {
-                symbol: if index == 0 {
-                   Some(trade.symbol.clone())
-                } else {
-                   None
-                },
-                date: buy_trade.conclusion_time.date,
-                quantity: (buy_trade.quantity * buy_trade.multiplier).normalize(),
-                price: (buy_price / buy_trade.multiplier).normalize(),
-                long_term_ownership: buy_trade.long_term_ownership_deductible.is_some(),
-                tax_free: buy_trade.tax_exemption_applied,
-            });
-        }
-
-        trades_table.add_row(TradeRow {
-            symbol: trade.symbol,
-            quantity: trade.quantity,
-            buy_price: (purchase_cost / trade.quantity).round_to(price_precision).normalize(),
-            sell_price,
-            commission,
-
-            revenue: details.revenue,
-            local_revenue: details.local_revenue,
-
-            profit: details.profit,
-            local_profit: details.local_profit,
-            taxable_local_profit: details.taxable_local_profit,
-
-            tax_to_pay: details.tax_to_pay,
-            tax_deduction: details.tax_deduction,
-
-            real_profit: details.real_profit_ratio.map(Cell::new_ratio),
-            real_tax: details.real_tax_ratio.map(Cell::new_ratio),
-            real_local_profit: details.real_local_profit_ratio.map(Cell::new_ratio),
-        });
-    }
-
-    if same_currency {
-        trades_table.hide_local_revenue();
-        trades_table.hide_local_profit();
-        trades_table.hide_real_tax();
-        trades_table.hide_real_local_profit();
-    }
-    if !long_term_ownership {
-        fifo_table.hide_long_term_ownership();
-    }
-    if !tax_exemptions {
-        trades_table.hide_taxable_local_profit();
-        trades_table.hide_tax_deduction();
-        fifo_table.hide_tax_free();
-    }
-
-    let (tax_year, _) = portfolio.tax_payment_day().get(execution_date, true);
-    let tax_to_pay = Cash::new(country.currency, country.tax_to_pay(
-        IncomeType::Trading, tax_year, total_taxable_local_profit.amount, None));
-
-    let mut totals = trades_table.add_empty_row();
-    totals.set_commission(total_commission);
-    totals.set_revenue(total_revenue);
-    totals.set_local_revenue(total_local_revenue);
-    totals.set_profit(total_profit);
-    totals.set_local_profit(total_local_profit);
-    totals.set_taxable_local_profit(total_taxable_local_profit);
-    totals.set_tax_to_pay(tax_to_pay);
-    totals.set_tax_deduction(total_tax_deduction);
-
-    trades_table.print("Sell simulation results");
-    fifo_table.print("FIFO details");
-
-    Ok(())
-}
+}
\ No newline at end of file
